

// Name
//===================================
//
// HydrantMovement
//


// DEPENDENCIES
//===================================

    #include "collision.h"
    #include "hydrant_movement.h"
    #include "sprite.h"
    #include "sprite_graphics.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    HydrantMovement::~HydrantMovement() {};

    void HydrantMovement::update( Sprite& sprite, Input& input, Camera& camera, Map& lvmap, SpriteGraphics* graphics, Game& game, EventSystem& events )
    {
        if ( sprite.isDead() )
            sprite.interact_ = false;

        if ( awake_ && delay_.done() )
        {
            if ( sprite.onGround() )
                sprite.jump();
        }
        else if ( delay_.counter() > 24 )
        {
            graphics->current_frame_x_ = 32;
        }
        else if ( delay_.counter() > 8 )
        {
            graphics->current_frame_x_ = 16;
        }

        if ( delay_.counter() % 4 == 1 )
        {
            graphics->current_frame_y_ = 16;
        }
        else
        {
            graphics->current_frame_y_ = 0;
        }
    };

    void HydrantMovement::interact( Collision& collision, Sprite& sprite, Sprite& other )
    {
        if ( other.hasType( Sprite::SpriteType::HERO ) )
        {
            if ( awake_ )
            {
                if ( delay_.done() )
                {
                    if ( other.xSubPixels() > sprite.hitboxRight() )
                        sprite.moveRight();
                    else
                        sprite.moveLeft();

                    if ( collision.collideAny() )
                        other.hurt();
                }
                else
                {
                    delay_.update();
                }
            }
            else
            {
                if ( other.hitboxRight() > sprite.xSubPixels() - 40000 && other.xSubPixels() < sprite.hitboxRight() + 40000 )
                {
                    awake_ = true;
                    delay_.start();
                }
            }
        }
        else if ( other.hasType( Sprite::SpriteType::ENEMY ) )
        {
            sprite.collideStopAny( collision );
        }
    };
