




// Name
//===================================
//
// Inventory
//


// DEPENDENCIES
//===================================

    #include "inventory.h"
    #include <iostream>


// STATIC PROPERTIES
//===================================

// METHODS
//===================================

    Inventory::Inventory( Clock* clock )
    :
        clock_ ( clock )
    {
    };

    Inventory::~Inventory()
    {
        if ( clock_ != nullptr )
        {
            delete clock_;
            clock_ = nullptr;
        }
    };

    void Inventory::init( Clock* clock )
    {
        if ( clock_ != nullptr )
        {
            delete clock_;
        }

        clock_ = clock;

        marquee_.x_ = Unit::WINDOW_WIDTH_PIXELS;
        marquee_.words_ = randomMarqueeMessage();

        total_funds_shown_ = total_funds_;
    };

    void Inventory::initLevelSelect( bool won )
    {
        if ( won )
        {
            total_funds_ += funds_;
        }

        if ( total_funds_ > TOTAL_FUNDS_MAX )
            total_funds_ = TOTAL_FUNDS_MAX;

        reset();
    };

    void Inventory::reset()
    {
        hearts_ = HEARTS_MAX;
        funds_ = 0;
        funds_shown_ = 0;
    };

    void Inventory::update()
    {
        if ( funds_shown_ < funds_ )
            funds_shown_+=25;

        if ( funds_shown_ > funds_ )
            funds_shown_+=25;

        marquee_.update();
        if ( marquee_.flipped() )
            marquee_.words_ = randomMarqueeMessage();

        if ( clock_ != nullptr )
            clock_->update();
    };

    void Inventory::updateLevelSelect()
    {
        if ( total_funds_shown_ < total_funds_ )
            total_funds_shown_ += 100;
        if ( total_funds_ - total_funds_shown_ < 100 )
            total_funds_shown_ = total_funds_;
    };

    void Inventory::render( Graphics& graphics )
    {
        graphics.renderRect( bg_pos_, 1 );

        gem_icon_gfx_.render( graphics, gem_icon_pos_, nullptr );
        Text::renderNumber( graphics, funds_shown_, FUNDS_X, VERTICAL_POS, 6, Text::FontShade::DARK_GRAY );

        int heart_x_mem = heart_icon_pos_.x;
        for ( int i = 0; i < hearts_; ++i )
        {
            heart_icon_gfx_.render( graphics, heart_icon_pos_, nullptr );
            heart_icon_pos_.x += 8;
        }
        heart_icon_pos_.x = heart_x_mem;

        marquee_.render( graphics );

        if ( clock_ != nullptr )
        {
            Text::renderText( graphics, Text::timeToString( Text::stringifyNum( clock_->secondsFromTotal() ), Text::stringifyNum( clock_->minutesFromTotalSeconds() ) ), CLOCK_X, VERTICAL_POS );
            clock_gfx_.render( graphics, clock_gfx_pos_, nullptr );
        }
    };

    void Inventory::addFunds( int n )
    {
        funds_ += n;

        if ( funds_ > FUNDS_MAX )
            funds_ = FUNDS_MAX;
    };

    int Inventory::funds() const
    {
        return funds_;
    };

    void Inventory::loseHeart()
    {
        --hearts_;
    };

    void Inventory::gainHeart()
    {
        ++hearts_;
    };

    int Inventory::hearts() const
    {
        return hearts_;
    };

    std::string Inventory::randomMarqueeMessage() const
    {
        int n = -1;
        do
        {
            n = rand() * marquee_messages_.size() / RAND_MAX;
        }
        while ( n >= marquee_messages_.size() );

        return marquee_messages_[ n ];
    };

    int Inventory::clockTime() const
    {
        return clock_->totalSeconds();
    };

    Clock* Inventory::clock()
    {
        return clock_;
    };

    void Inventory::renderLevelSelect( Graphics& graphics )
    {
        gem_icon_gfx_.render( graphics, lv_select_gem_icon_pos_, nullptr );
        Text::renderNumber( graphics, total_funds_shown_, FUNDS_X, VERTICAL_POS_LEVEL_SELECT, 9 );
    };

    int Inventory::maxHP() const
    {
        return HEARTS_MAX;
    };
