


// Name
//===================================
//
// LevelSelectState
//


// DEPENDENCIES
//===================================

    #include "game.h"
    #include "input.h"
    #include "level_state.h"
    #include "level_select_state.h"
    #include "title_state.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    LevelSelectState::LevelSelectState( EventSystem* events, Inventory* inventory )
    :
        GameState ( StateID::LEVEL_SELECT_STATE )
    {
        int X = 32;

        for ( int i = 0; i < Level::LevelName::LV_LIMIT; ++i )
        {
            levels_.push_back( Text( Level::NameOLevel( i ), X, START_Y+(8*i) ) );
        }

        if ( events != nullptr )
        {
            events_ = events;
        }
        else
            events_ = new EventSystem;

        if ( inventory != nullptr )
            inventory_ = inventory;
        else
            inventory_ = new Inventory;
    };

    LevelSelectState::~LevelSelectState() {};

    void LevelSelectState::update( Game& game, Input& input, Graphics& graphics )
    {
        inventory_->updateLevelSelect();

        highlight_rect_.y = START_Y + ( 8 * selection_.value() );

        if ( input.pressed( Input::Action::MOVE_UP ) )
        {
            --selection_;
        }
        if ( input.pressed( Input::Action::MOVE_DOWN ) )
        {
            ++selection_;
        }

        if ( input.pressed( Input::Action::CONFIRM ) )
        {
            game.changeState( std::unique_ptr<GameState> ( new LevelState( *events_, *inventory_, (Level::LevelName)selection_.value() ) ) );
        }
        else if ( input.pressed( Input::Action::MENU ) )
        {
            game.changeState( std::unique_ptr<GameState> ( new TitleState() ) );
        }
    };

    void LevelSelectState::stateRender( Graphics& graphics )
    {
        title_.render( graphics );
        graphics.renderRect( highlight_rect_, 6 );

        int i = 0;
        for ( auto& l : levels_ )
        {
            Text::FontShade shade = Text::FontShade::BLACK;

            if ( i == selection_.value() )
                shade = Text::FontShade::WHITE;

            l.render( graphics, nullptr, shade );

            ++i;
        }

        if ( inventory_ != nullptr )
            inventory_->renderLevelSelect( graphics );
    };

    void LevelSelectState::init( Game& game, Graphics& graphics )
    {
        inventory_->initLevelSelect( events_->won_ );
        events_->reset();
    };

    void LevelSelectState::backFromPop( Game& game, Graphics& graphics ) {};
