


// Name
//===================================
//
// PlayerMovement
//


// DEPENDENCIES
//===================================

    #include "block.h"
    #include "camera.h"
    #include "collision.h"
    #include "game.h"
    #include "event_system.h"
    #include "input.h"
    #include "inventory.h"
    #include "level.h"
    #include "map.h"
    #include "pause_state.h"
    #include "player_movement.h"
    #include "sprite.h"
    #include "sprite_graphics.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    PlayerMovement::~PlayerMovement() {};

    void PlayerMovement::update( Sprite& sprite, Input& input, Camera& camera, Map& lvmap, SpriteGraphics* graphics, Game& game, EventSystem& events )
    {
        if ( !events.won_ )
        {
            if ( input.pressed( Input::Action::MENU ) )
            {
                sprite.jump_lock_ = true;
                game.pushState( std::unique_ptr<GameState> ( new PauseState( lvmap.palette(), events ) ) );
            }
            if ( input.held( Input::Action::MOVE_DOWN ) )
            {
                sprite.duck();
            }
            else
            {
                sprite.unduck();
            }

            if ( input.held( Input::Action::MOVE_LEFT ) && !input.held( Input::Action::MOVE_RIGHT ) && ( !sprite.isDucking() || !sprite.onGround() ) )
            {
                sprite.moveLeft();
            }

            if ( input.held( Input::Action::MOVE_RIGHT ) && !input.held( Input::Action::MOVE_LEFT ) && ( !sprite.isDucking() || !sprite.onGround() ) )
            {
                sprite.moveRight();
            }

            if ( !input.held( Input::Action::MOVE_RIGHT ) && !input.held( Input::Action::MOVE_LEFT ))
            {
                if ( sprite.isDucking() )
                    sprite.stopDucking();
                else
                    sprite.stop();
            }

            if ( input.held( Input::Action::RUN ) )
            {
                sprite.run();
            }

            if ( input.held( Input::Action::EXTRA1 ) )
            {
                camera.moveLeft();
            }

            if ( input.held( Input::Action::EXTRA2 ) )
            {
                camera.moveRight();
            }

            if ( input.held( Input::Action::RUN ) )
            {
                sprite.run();
            }

            if ( input.held( Input::Action::JUMP ) )
            {
                    sprite.jump();
                    sprite.jump_lock_ = true;

                sprite.slowFall();
            }

            if ( input.released( Input::Action::JUMP ) )
            {
                sprite.jump_lock_ = false;
            }

            if ( input.held( Input::Action::MOVE_UP ) || input.held( Input::Action::MOVE_DOWN ) )
            {
                if ( sprite.touching_ladder_ )
                    sprite.grabLadder();
            }

            if ( input.held( Input::Action::MOVE_UP ) )
            {
                sprite.moveUp();

                if ( sprite.onGround() && !sprite.isMoving() )
                    camera.moveUp();
            }

            if ( input.released( Input::Action::MOVE_UP ) )
            {
                sprite.stopLookingUp();
            }

            if ( input.held( Input::Action::MOVE_DOWN ) )
            {
                sprite.moveDown();

                if ( sprite.onGround() && !sprite.isMoving() )
                {
                    camera.moveDown();
                }
            }
        }

        // In case player gets stuck 'tween 2 blocks with just 1 empty block 'tween them vertically.
        // Makes player automatically move to the side till out o' space ala Super Mario Bros. 3.
        if ( sprite.collide_top_prev_ && sprite.collide_bottom_prev_ && !sprite.isDucking() )
        {
            if ( sprite.directionX() == Direction::Horizontal::RIGHT )
                sprite.moveRight();
            else
                sprite.moveLeft();
        }

        if ( !sprite.touching_ladder_ )
            sprite.releaseLadder();

        sprite.boundaries( camera );
    };

    void PlayerMovement::interact( Collision& collision, Sprite& sprite, Sprite& other )
    {/*
        if ( other.hasType( Sprite::SpriteType::ENEMY ) )
            if ( collision.collideAny() && !collision.collideBottom() )
                sprite.hurt();
            else if ( collision.collideBottom() )
            {
                sprite.collideStopYBottom( collision.overlapYBottom() );
                sprite.jump();
                other.kill();
            }*/
    };

/*
    void PlayerMovement::collision( Collision collision, Sprite& sprite, Block& block, Level& level, EventSystem& events, Inventory& inventory, Camera& camera, GameState* state, Graphics& graphics )
    {
        if ( block.hasType( BlockType::BlockAction::SOLID ) )
        {
            if ( !sprite.collide_bottom_prev_ || !sprite.collide_top_prev_ )
            {
                if ( collision.collideLeft() )
                {
                    sprite.collideStopXLeft( collision.overlapXLeft() );
                }

                if ( collision.collideRight() )
                {
                    sprite.collideStopXRight( collision.overlapXRight() );
                }
            }

            if ( collision.collideBottom() )
            {
                sprite.collideStopYBottom( collision.overlapYBottom() );
                sprite.collide_top_ = true;
            }

            if ( collision.collideTop() )
            {
                sprite.collide_bottom_ = true;

                if ( !sprite.collide_top_prev_ )
                {
                    sprite.collideStopYTop( collision.overlapYTop() );
                }
            }
        }

        if ( block.hasType( BlockType::BlockAction::FLAT_TOP ) )
        {

            // FLAT_TOP needs extra condition to prevent being bounced up to the top from jumping below & stopping just before the top.
            // However, needs 2000 subpixels padding to stop ducking from causing you to fall through.
            if ( collision.collideBottom() && sprite.hitboxPrevBottom() <= block.ySubPixels() + 1000 )
            {
                sprite.collideStopYBottom( collision.overlapYBottom() );
                sprite.collide_top_ = true;
            }
        }

        if ( block.hasType( BlockType::BlockAction::CLIMBABLE ) )
        {

            if ( collision.collideAny() )
            {
                sprite.touching_ladder_ = true;
            }
        }

        if ( block.hasType( BlockType::BlockAction::GEM ) )
        {

            if ( collision.collideAny() )
            {
                inventory.addFunds();
                level.currentMap().deleteBlock( block.location() );
            }
        }

        if ( block.hasType( BlockType::BlockAction::HURT ) )
        {

            if ( collision.collideAny() )
            {
                hurt( sprite, inventory );
            }
        }

        if ( block.hasType( BlockType::BlockAction::HURT_TOP ) )
        {

            if ( collision.collideBottom() )
            {
                hurt( sprite, inventory );
            }
        }

        if ( block.hasType( BlockType::BlockAction::GOAL ) )
        {

            if ( collision.collideAny() )
            {
                level.currentMap().deleteBlock( block.location() );
                events.won_ = true;
            }
        }

        if ( block.hasType( BlockType::BlockAction::WARP1 ) )
        {

            if ( collision.collideAny() )
            {
                level.warp( 0, sprite, camera, state, graphics );
            }
        }

        if ( block.hasType( BlockType::BlockAction::WARP2 ) )
        {

            if ( collision.collideAny() )
            {
                level.warp( 1, sprite, camera, state, graphics );
            }
        }

        if ( block.hasType( BlockType::BlockAction::WARP3 ) )
        {

            if ( collision.collideAny() )
            {
                level.warp( 2, sprite, camera, state, graphics );
            }
        }

        if ( block.hasType( BlockType::BlockAction::WARP4 ) )
        {

            if ( collision.collideAny() )
            {
                level.warp( 3, sprite, camera, state, graphics );
            }
        }

        if ( block.hasType( BlockType::BlockAction::WARP5 ) )
        {

            if ( collision.collideAny() )
            {
                level.warp( 4, sprite, camera, state, graphics );
            }
        }
    };

    void PlayerMovement::hurt( Sprite& sprite, Inventory& inventory )
    {
        if ( !invincible_ )
        {
            inventory.loseHeart();

            if ( inventory.hearts() < 1 )
            {
                sprite.kill();
            }
            else
            {
                invincible_ = true;
                invincibility_timer_.start();
            }
        }
    };
*/
